﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using DLW.Properties;

namespace DLW
{
    /// <summary>
    /// アイコンデザイン設定情報を保持し、描画するクラス
    /// </summary>
    class IconDesign
    {
        //配色配列index対応定義
        const int CBack = 0;
        const int CRead = 1;
        const int CWrite = 2;
        const int CReadN = 3;
        const int CWriteN = 4;
        const int CFont = 5;
        const int CFontVoid = 6;

        const int CLockCaps = 7;
        const int CLockScroll = 8;
        const int CLockNum = 9;
        const int CLockOff = 10;
        const int CLockBg = 11;

        /// <summary>配色設定</summary>
        internal readonly Color[] Colors = new Color[]
        {
            Settings.Default.CBack,
            Settings.Default.CRead,
            Settings.Default.CWrite,
            Settings.Default.CReadN,
            Settings.Default.CWriteN,
            Settings.Default.CFont,
            Settings.Default.CFontVoid,
            Settings.Default.CLockCaps,
            Settings.Default.CLockScroll,
            Settings.Default.CLockNum,
            Settings.Default.CLockOff,
            Settings.Default.CLockBg,
        };
        /// <summary>フォント設定</summary>
        internal Font[] Fonts = new Font[]
        {   //[0]=Drive, [1]=Network, [2]=KeyLock(readonly), [3]=Disk(Driveに下線を追加)
            Settings.Default.Font,
            Settings.Default.FontN,
            Settings.Default.FontL,
            new Font(Settings.Default.Font, (Settings.Default.Font.Style | FontStyle.Underline))
        };
        /// <summary>フォント陰影設定</summary>
        internal int FontShadow = Settings.Default.FontShadow;
        /// <summary>アイコン陰影設定</summary>
        internal int IconShadow = Settings.Default.IconShadow;

        /// <summary>色設定名を返す</summary>
        internal string[] selColor()
        {
            return new string[]
            {
                "通常背景",
                "Disk読込",
                "Disk書込",
                "LAN受信",
                "LAN送信",
                "アイコン文字",
                "無効時文字",
                "Capsキー",
                "Scrollキー",
                "Numキー",
                "キーOff文字",
                "キー背景",
            };
        }
        /// <summary>フォント設定名を返す</summary>
        internal string[] selFont()
        {
            return new string[]
            {   
                "Drive名",
                "LAN種類",
                //"KeyLock", //KeyLockは変更不可
            };
        }
        /// <summary>色設定・フォント設定を保存する</summary>
        internal void Save()
        {
            //色設定保存
            Settings.Default.CBack = Colors[CBack];
            Settings.Default.CRead = Colors[CRead];
            Settings.Default.CWrite = Colors[CWrite];
            Settings.Default.CReadN = Colors[CReadN];
            Settings.Default.CWriteN = Colors[CWriteN];
            Settings.Default.CFont = Colors[CFont];
            Settings.Default.CFontVoid = Colors[CFontVoid];
            Settings.Default.CLockCaps = Colors[CLockCaps];
            Settings.Default.CLockScroll = Colors[CLockScroll];
            Settings.Default.CLockNum = Colors[CLockNum];
            Settings.Default.CLockOff = Colors[CLockOff];
            Settings.Default.CLockBg = Colors[CLockBg];
            //フォント設定保存
            Settings.Default.Font = Fonts[0];
            Settings.Default.FontN = Fonts[1];
            //Settings.Default.FontL = Fonts[2]; //KeyLockは変更不可
        }

        /// 描画セクション======================================================
        /// <summary>現在のDPIスケールにおける小アイコンサイズでBitmapを生成</summary>
        internal static Bitmap Bitmap()
        {
            return new Bitmap(
                GetSystemMetrics(SM_CXSMICON),
                GetSystemMetrics(SM_CYSMICON));
        }
        /// <summary>あるべき小アイコンのサイズを把握するための関数</summary>
        [DllImport("user32")]
        internal static extern int GetSystemMetrics(int systemMetric);
        internal const int SM_CXSMICON = 49;
        internal const int SM_CYSMICON = 50;

        /// <summary>アイコン一括生成</summary>
        /// <param name="start">初期表示(無効状態アイコンのみ生成)か</param>
        /// <param name="drive"></param>
        /// <param name="underline"></param>
        /// <param name="driveR"></param>
        /// <param name="underlineR"></param>
        /// <returns></returns>
        internal Icon[][] Get(bool start, char drive, bool underline, char driveR, bool underlineR)
        {
            //アイコン表示パターンの総数(起動用のものは１個だけ、全パターン用のものは必要なバリエーション全て）
            int PATTERNS = (start ? 1 : 10);
            Icon[][] ret = new Icon[PATTERNS][];
            using (Bitmap b = IconDesign.Bitmap())
            {
                if (drive > 0)
                {
                    for (int i1 = 0; i1 < PATTERNS; i1++) //左ドライブのパターンを生成
                    {
                        Icon[] icons = new Icon[PATTERNS];
                        for (int i2 = 0; i2 < PATTERNS; i2++) //右ドライブのパターンも作成
                        {
                            DrawItem(b, drive, 0, i1, underline);
                            DrawItem(b, driveR, b.Width / 2, i2, underlineR);
                            icons[i2] = Icon.FromHandle(b.GetHicon());
                        }
                        ret[i1] = icons;
                    }
                }
                else
                {   //1ドライブのみでパターンを生成
                    Icon[] icons = new Icon[PATTERNS];
                    for (int i2 = 0; i2 < PATTERNS; i2++)
                    {
                        DrawItem(b, driveR, b.Width / 2, i2, underlineR);
                        icons[i2] = Icon.FromHandle(b.GetHicon());
                    }
                    ret[0] = icons;
                }
            }
            return ret;
        }
        /// <summary>アイコン描画</summary>
        /// <param name="bitmap">描画対象</param>
        /// <param name="drive"></param>
        /// <param name="x"></param>
        /// <param name="pattern"></param>
        internal void DrawItem(Bitmap bitmap, char drive, int x, int pattern, bool underline)
        {
            //LANか否かでR/W表示色を変える
            bool isLAN = (drive >= 'a' && drive <= 'z');
            Color rd = Colors[(isLAN ? CReadN : CRead)];
            Color wr = Colors[(isLAN ? CWriteN : CWrite)];

            using (Graphics g = Graphics.FromImage(bitmap))
            {
                int boxW = bitmap.Width / 2;
                int boxH = bitmap.Height;
                int lineW = 1; // 8→1, 10→1, 12→1
                int lineH = (int)Math.Ceiling((double)(boxH - 2) / 7); // 16→2, 20→2, 24→3

                //文字などがはみ出さないよう対策(ver1.2.3)
                g.SetClip(new Rectangle(x, 0, boxW, boxH));

                if (drive == Watcher.WKeylock)
                {   //背景の描画(スクロールロック)
                    using (Brush brush = new SolidBrush(Colors[CLockBg]))
                    {
                        g.FillRectangle(brush, x, 0, boxW, boxH);
                    }
                    //文字の描画
                    DrawString(g, "c", x, boxW - 1, boxH, boxH * 7 / 20,
                        Colors[((pattern & 1) == 1 ? CLockCaps : CLockOff)],
                        Fonts[2], ((pattern & 1) == 1));
                    DrawString(g, "n", x, boxW - 1, boxH, boxH - 1,
                        Colors[((pattern & 2) == 2 ? CLockNum : CLockOff)],
                        Fonts[2], ((pattern & 2) == 2));
                    DrawString(g, "s", x + 2, boxW - 1, boxH, boxH * 13 / 20,
                        Colors[((pattern & 4) == 4 ? CLockScroll : CLockOff)],
                        Fonts[2], ((pattern & 4) == 4));
                    if ((pattern & 2) == 2)
                    {
                        DrawString(g, "n", x, boxW - 1, boxH, boxH - 1,
                            Colors[CLockNum], Fonts[2], true);
                    }
                }
                else
                {   //背景の描画(読込:3/6/9、書込:7/8/9)
                    using (Brush brush = new SolidBrush(
                        (pattern == 3 || pattern == 6) ? rd
                        : (pattern >= 7) ? wr
                        : Colors[CBack]))
                    {
                        g.FillRectangle(brush, x, 0, boxW, boxH);
                    }
                    if (pattern == 9)
                    {   //RW同時発生中(上半分に読込色を書き足し)
                        using (Brush brush = new SolidBrush(rd))
                        {
                            g.FillRectangle(brush, x, 0, boxW, boxH / 2);
                        }
                    }
                    //ドライブ文字の描画（無効ドライブは無効色で描画）
                    DrawString(g, drive.ToString(), x, boxW, boxH, 0,
                        (pattern != 0 ? Colors[CFont] : Colors[CFontVoid]),
                        (drive >= '0' && drive <= 'Z' ? (underline ? Fonts[3] : Fonts[0]) : Fonts[1]),
                        (pattern != 0));
                    //外枠の描画(読込:2/3/5/6/8/9、書込:4/5/6/7/8/9)
                    if (pattern >= 2 && pattern <= 9)
                    {   //上半分の直近アクセス線を描画
                        using (Brush brush = new SolidBrush((pattern != 4 && pattern != 7) ? rd : wr))
                        {
                            g.FillRectangle(brush, x, 0, boxW, lineH);
                            g.FillRectangle(brush, x, 0, lineW, boxH / 2);
                            g.FillRectangle(brush, x + boxW - lineW, 0, lineW, boxH / 2);
                        }
                        //下半分の直近アクセス線を描画
                        using (Brush brush = new SolidBrush((pattern == 2 || pattern == 3) ? rd : wr))
                        {
                            g.FillRectangle(brush, x, boxH / 2, lineW, boxH / 2);
                            g.FillRectangle(brush, x + boxW - lineW, boxH / 2, lineW, boxH / 2);
                            g.FillRectangle(brush, x, boxH - lineH, boxW, lineH);
                        }
                    }
                }
                //立体感付け
                using (Pen pen = new Pen(Color.FromArgb(Math.Abs(IconShadow), IconShadow < 0 ? Color.Black : Color.White)))
                {
                    g.DrawLine(pen, x, 0, x + boxW - 2, 0);
                    g.DrawLine(pen, x, 0, x, boxH - 2);
                }
                using (Pen pen = new Pen(Color.FromArgb(Math.Abs(IconShadow), IconShadow < 0 ? Color.White : Color.Black)))
                {
                    g.DrawLine(pen, x + 1, boxH - 1, x + boxW - 1, boxH - 1);
                    g.DrawLine(pen, x + boxW - 1, 1, x + boxW - 1, boxH - 1);
                }

                g.ResetClip();
            }
        }

        //内部文字描画
        private void DrawString(Graphics g, string text, int x, int w, int height, int yOffset, Color c, Font f, bool FontShadow)
        {
            //描画フォーマットを微調整
            StringFormat flags = new StringFormat();
            flags.Alignment = StringAlignment.Center;
            flags.LineAlignment = StringAlignment.Near;
            flags.FormatFlags = StringFormatFlags.NoWrap;
            //フォントサイズに応じた描画位置y座標を計算で算出
            int emheight = f.FontFamily.GetEmHeight(f.Style);
            int lineheight = f.FontFamily.GetLineSpacing(f.Style);
            int descent = f.FontFamily.GetCellDescent(f.Style);
            float height2 = f.SizeInPoints * g.DpiY / 72 * (1 + ((float)(lineheight - emheight + descent) / emheight / 2));
            int y = (height / 2 - (int)Math.Round((height2 + 1) / 2));
            if (y < 2) { y = 2; }
            if (yOffset > 0)
            {   //文字を集約表示する際のロジックは暫定版：それなりに計算
                y = yOffset - (int)(f.SizeInPoints * g.DpiY / 72 * f.FontFamily.GetCellAscent(f.Style) / emheight);
            }
            //文字影を描画する(ただし無効文字・影無しの場合は省略)
            if (FontShadow)
            {
                //文字色をもとに、影の明るさを計算する
                Color shadowColor = (c.R * 30 + c.G * 59 + c.B * 11 > 12800 ? Color.Black : Color.White);
                using (Brush brush = new SolidBrush(Color.FromArgb(this.FontShadow * 3 / 4, shadowColor)))
                {
                    g.DrawString(text, f, brush, new RectangleF(x, y + 1, w, height), flags);
                    g.DrawString(text, f, brush, new RectangleF(x + 1, y + 1, w, height), flags);
                }
            }
            //文字本体を描画する
            using (Brush brush = new SolidBrush(c))
            {
                g.DrawString(text, f, brush, new RectangleF(x, y, w, height), flags);
            }
        }
    }
}
