﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using DLW.Properties;

namespace DLW
{
    public partial class MainForm : Form
    {
        //アイコンプレビュー表示用クラス
        private class Monitor : Watcher
        {
            Random Random;
            private volatile new bool rd = false;
            private volatile new bool wr = false;

            internal Monitor(char drive, int rdCount, int wrCount)
                : base(drive)
            {
                Random = new Random((int)drive);
                base.rdCount = rdCount;
                base.wrCount = wrCount;
            }
            //表示アイコン種類決定：ランダムに点滅させる
            internal int Check(int TTick, int TDelay)
            {
                int tick = Environment.TickCount;
                if (this.rd)
                {
                    if (Random.Next(TTick * 3 + 20) < TTick)
                    {
                        base.rd = tick;
                    }
                }
                else if (tick - base.rd > TTick * 2 + TDelay && Random.Next(1000) < base.rdCount)
                {
                    base.rd = tick;
                }
                if (this.wr)
                {
                    if (Random.Next(TTick * 3 + 20) < TTick)
                    {
                        base.wr = tick;
                    }
                }
                else if (tick - base.wr > TTick * 2 + TDelay && Random.Next(1000) < base.wrCount)
                {
                    base.wr = tick;
                }
                this.rd = (tick == base.rd);
                this.wr = (tick == base.wr);
                return base.Check(tick);
            }
        }

        //配色・フォント設定
        IconDesign IconDesign; //loadSettingで生成 = new IconDesign();
        //プレビュー表示用
        Bitmap icon = IconDesign.Bitmap();
        Monitor drive = new Monitor('D', 120, 80);
        Monitor driveR = new Monitor('l', 90, 110);
        Timer Timer;
        //※ComboBox右クリックによるダイアログ表示は、
        //  タイマイベント内でMouseButtonを調べることで強引に実現している。修正時注意！

        //コンストラクタ：アイコン/タイトルの設定のみ行う
        public MainForm()
        {
            InitializeComponent();
            try
            {
                this.Icon = Program.Icon;
            }
            catch (Exception) { }
            this.Text = Program.FileDescription;
        }

        //画面が閉じた際に
        protected override void OnClosed(EventArgs e)
        {   //タイマを確実に停止する
            Timer.Dispose();
            //アイコン画像を開放する
            Bitmap icon = this.icon;
            this.icon = null;
            icon.Dispose();
            drive.Dispose();
            driveR.Dispose();
            //
            base.OnClosed(e);
        }

        //ロード時処理
        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);
            //ショートカットの初期設定
            getShortcutStateIntoCheckbox(chkShortcutDesktop);
            getShortcutStateIntoCheckbox(chkShortcutStartup);
            getShortcutStateIntoCheckbox(chkShortcutQuickLaunch);
            //キーロックコンボの選択肢設定
            string[] values = { "(制御なし)", "ON固定", "OFF固定" };
            CapsLock.Items.AddRange(values);
            ScrollLock.Items.AddRange(values);
            NumLock.Items.AddRange(values);
            //パーティション集約表示の選択肢設定
            string[] values2 = { "先頭ドライブ名表示", "１開始連番表示", "物理ディスク番号表示" };
            selAggregateCaption.Items.AddRange(values2);
            //ドライブのカスタム設定
            WDisk.CheckedChanged += CheckedChanged;
            WRemovable.CheckedChanged += CheckedChanged;
            WRamDisk.CheckedChanged += CheckedChanged;
            txtCustom.TextChanged += CheckedChanged;
            txtOmitDrive.TextChanged += CheckedChanged;
            optCustom.CheckedChanged += CheckedChanged;
            //ネットワーク(別表示)
            WSpecifiedLAN.CheckedChanged += (sender, e2) =>
            {
                SpecifiedLanName.Enabled = WSpecifiedLAN.Checked;
                if (Timer != null && SpecifiedLanName.Enabled) { SpecifiedLanName.Focus(); }
            };
            //別表示ネットワーク名の選択肢既定値設定
            SpecifiedLanName.Items.AddRange(new string[] { "VMnet", "Virtual", "Hyper-V", "VPN", "PPP" });
            //キーロックのカスタム設定
            WKeylock.CheckedChanged += CheckedChanged2;
            XCapsLock.CheckedChanged += CheckedChanged2;
            XScrLk.CheckedChanged += CheckedChanged2;
            XNumLk.CheckedChanged += CheckedChanged2;
            //パーティション集約
            WAggregateDisk.CheckedChanged += (sender, e2) =>
            {
                selAggregateCaption.Enabled = WAggregateDisk.Checked;
                if (Timer != null && selAggregateCaption.Enabled) { selAggregateCaption.Focus(); }
            };
            //タイミング設定
            TTick.ValueChanged += (sender, e2) =>
            {
                if (Timer != null)
                {
                    Timer.Interval = (int)TTick.Value;
                }
            };
            //色設定（描画・ボタン押下イベント設定）
            selColor.DrawMode = DrawMode.OwnerDrawFixed;
            selColor.DrawItem += (sender, e2) =>
            {
                //背景を描画する
                e2.DrawBackground();
                if ((e2.State & DrawItemState.Focus) == DrawItemState.Focus)
                {
                    e2.DrawFocusRectangle();
                }
                if (e2.Index > -1)
                {   //色設定を描画する（外枠＝白固定）
                    using (Brush brush = new SolidBrush(IconDesign.Colors[e2.Index]))
                    using (Pen pen = new Pen(Color.White))
                    {
                        e2.Graphics.FillRectangle(brush, e2.Bounds.Left, e2.Bounds.Top, e2.Bounds.Height - 1, e2.Bounds.Height - 1);
                        e2.Graphics.DrawRectangle(pen, e2.Bounds.Left, e2.Bounds.Top, e2.Bounds.Height - 1, e2.Bounds.Height - 1);
                    }
                    //文字を描画する
                    string txt = ((ComboBox)sender).Items[e2.Index].ToString();
                    using (Brush brush = new SolidBrush(e2.ForeColor))
                    {
                        e2.Graphics.DrawString(txt, e2.Font, brush, e2.Bounds.Left + e2.Bounds.Height, e2.Bounds.Top);
                    }
                    return;
                }
            };
            btnColor.Click += ColorDialog;

            //フォント設定（描画・ボタン押下イベント設定）
            selFont.DrawMode = DrawMode.OwnerDrawFixed;
            selFont.DrawItem += (sender, e2) =>
            {
                //背景を描画する
                e2.DrawBackground();
                if ((e2.State & DrawItemState.Focus) == DrawItemState.Focus)
                {
                    e2.DrawFocusRectangle();
                }
                if (e2.Index > -1)
                {   //文字を（フォント設定どおりに）描画する
                    string txt = ((ComboBox)sender).Items[e2.Index].ToString();
                    using (Brush brush = new SolidBrush(e2.ForeColor))
                    {
                        e2.Graphics.DrawString(txt, IconDesign.Fonts[e2.Index], brush, e2.Bounds);
                    }
                    return;
                }
            };
            btnFont.Click += FontDialog;

            //ショートカット設定イベント
            chkShortcutDesktop.Click += chkShortcut_Click;
            chkShortcutStartup.Click += chkShortcut_Click;
            chkShortcutQuickLaunch.Click += chkShortcut_Click;
            //保存イベント等
            btnOk.Click += (sender, e2) =>
            {
                Save();
                Program.Refresh("表示設定を更新しました。");
                this.Close();
            };
            btnCancel.Click += (sender, e2) =>
            {
                this.Close();
            };
            btnApply.Click += (sender, e2) =>
            {
                Save();
                Program.Refresh("表示設定を更新しました。");
            };
            btnReset.Click += (sender, e2) =>
            {
                Settings.Default.Reset();
                Settings.Default.Version = Application.ProductVersion;
                Settings.Default.Save();
                Program.Refresh("表示設定をリセットしました。");
                loadSettings();
            };
            //設定値読み込み（イベント設定後に実施）
            loadSettings();
            //タイマ起動
            Timer = new Timer();
            Timer.Tick += DrawItem;
            Timer.Interval = (int)TTick.Value;
            Timer.Start();
        }
        //設定読み込み
        private void loadSettings()
        {
            //ドライブ表示対象
            optAuto.Checked = !Watcher.Contains(Watcher.WCustom);
            WDisk.Checked = Watcher.Contains(Watcher.WDisk);
            WRemovable.Checked = Watcher.Contains(Watcher.WRemovable);
            WRamDisk.Checked = Watcher.Contains(Watcher.WRamDisk);
            txtOmitDrive.Text = Settings.Default.TargetOmit;
            txtCustom.Text = Settings.Default.TargetCustom;
            optCustom.Checked = Watcher.Contains(Watcher.WCustom);
            CheckedChanged(optCustom, null);//ここでイベントを強制発動
            //LAN表示対象。設定変更する前にネットワーク一覧をリフレッシュする
            Network.Refresh(null, null);
            WEthernet.Checked = Watcher.Contains(Watcher.WEthernet);
            WEthernet.Enabled = (Network.Check(Watcher.WEthernet));
            WWireless.Checked = Watcher.Contains(Watcher.WWireless);
            WWireless.Enabled = (Network.Check(Watcher.WWireless));
            IgnoreBroadcastRx.Checked = Settings.Default.IgnoreBroadcastRx;
            //一部LAN別表示
            WSpecifiedLAN.Checked = Watcher.Contains(Watcher.WSpecifiedLAN);
            SpecifiedLanName.Text = Settings.Default.SpecifiedLanName;
            //キーロック関連
            WKeylock.Checked = Watcher.Contains(Watcher.WKeylock);
            XCapsLock.Checked = Watcher.Contains(Watcher.XCapsLock);
            XScrLk.Checked = Watcher.Contains(Watcher.XScrLk);
            XNumLk.Checked = Watcher.Contains(Watcher.XNumLk);
            CapsLock.SelectedIndex = Watcher.IndexOf(Watcher.XCapsLock);
            ScrollLock.SelectedIndex = Watcher.IndexOf(Watcher.XScrLk);
            NumLock.SelectedIndex = Watcher.IndexOf(Watcher.XNumLk);
            CheckedChanged2(optCustom, null);//すべてチェックoffの場合でもここでイベントを強制発動
            //表示
            TTick.Value = Settings.Default.TTick;
            TDelay.Value = Settings.Default.TDelay;
            TRefresh.Value = Settings.Default.TRefresh;
            TAutoRefresh.Checked = Settings.Default.TAutoRefresh;
            //パーティション集約表示
            WAggregateDisk.Checked = Watcher.Contains(Watcher.WAggregateDisk);
            selAggregateCaption.SelectedIndex = Watcher.IndexOf(Watcher.WAggregateDisk);
            //色・フォント
            IconDesign = new IconDesign();
            selColor.Items.Clear();
            selColor.Items.AddRange(IconDesign.selColor());
            selColor.SelectedIndex = 0;
            selFont.Items.Clear();
            selFont.Items.AddRange(IconDesign.selFont());
            selFont.SelectedIndex = 0;
            FontShadow.Value = Math.Abs(Settings.Default.FontShadow); //過去バージョンでマイナス値を許容していたため、絶対値に変換
            IconShadow.Value = Settings.Default.IconShadow;
        }
        //設定保存
        private void Save()
        {
            //一部LAN名未入力ならばチェックを強制解除する
            if (string.IsNullOrEmpty(SpecifiedLanName.Text.Trim())) { WSpecifiedLAN.Checked = false; }
            //ドライブ/LAN表示対象設定
            Settings.Default.Target = createTargetText();
            Settings.Default.TargetOmit = txtOmitDrive.Text;
            if (optCustom.Checked)
            {
                Settings.Default.TargetCustom = txtCustom.Text;
            }
            Settings.Default.IgnoreBroadcastRx = IgnoreBroadcastRx.Checked;
            if (WSpecifiedLAN.Checked)
            {   //別表示するLANの設定も上書き
                Settings.Default.SpecifiedLanName = SpecifiedLanName.Text;
            }
            //表示設定
            Settings.Default.TTick = (ushort)TTick.Value;
            Settings.Default.TDelay = (ushort)TDelay.Value;
            Settings.Default.TRefresh = (ushort)TRefresh.Value;
            Settings.Default.TAutoRefresh = TAutoRefresh.Checked;
            //色設定・フォント設定
            IconDesign.Save();
            Settings.Default.FontShadow = (ushort)FontShadow.Value;
            Settings.Default.IconShadow = (short)IconShadow.Value;
            //設定保存
            Settings.Default.Version = Application.ProductVersion;
            Settings.Default.Save();
            //キーロック設定を反映
            KeyLock.loadSettings = true;
        }
        //表示対象テキストを生成
        private String createTargetText()
        {
            StringBuilder sb = new StringBuilder();
            if (optCustom.Checked) { sb.Append(Watcher.WCustom); }
            if (WDisk.Checked) { sb.Append(Watcher.WDisk); }
            if (WRemovable.Checked) { sb.Append(Watcher.WRemovable); }
            if (WRamDisk.Checked) { sb.Append(Watcher.WRamDisk); }
            if (WEthernet.Checked) { sb.Append(Watcher.WEthernet); }
            if (WWireless.Checked) { sb.Append(Watcher.WWireless); }
            if (WSpecifiedLAN.Checked) { sb.Append(Watcher.WSpecifiedLAN); }
            if (WKeylock.Checked) { sb.Append(Watcher.WKeylock); }
            if (XCapsLock.Checked)
            {
                sb.Append(Watcher.XCapsLock);
                sb.Append(CapsLock.SelectedIndex.ToString());
            }
            if (XScrLk.Checked)
            {
                sb.Append(Watcher.XScrLk);
                sb.Append(ScrollLock.SelectedIndex.ToString());
            }
            if (XNumLk.Checked)
            {
                sb.Append(Watcher.XNumLk);
                sb.Append(NumLock.SelectedIndex.ToString());
            }
            if (WAggregateDisk.Checked)
            {
                sb.Append(Watcher.WAggregateDisk);
                sb.Append(selAggregateCaption.SelectedIndex.ToString());
            }
            return sb.ToString();
        }
        #region 汎用イベント
        /// <summary>ドライブ表示対象設定チェックボックス変更時の表示制御</summary>
        private void CheckedChanged(object sender, EventArgs e)
        {
            WDisk.Enabled = optAuto.Checked;
            WRemovable.Enabled = optAuto.Checked;
            WRamDisk.Enabled = optAuto.Checked;
            lblOmitDrive.Enabled = optAuto.Checked;
            txtOmitDrive.Enabled = optAuto.Checked;
            txtCustom.Enabled = optCustom.Checked;
            if (sender is TextBox)
            {   //小文字→大文字補正
                TextBox tb = sender as TextBox;
                string oldText = tb.Text;
                string text = oldText.ToUpper();
                if (text != oldText)
                {
                    int s1 = tb.SelectionStart;
                    int s2 = tb.SelectionLength;
                    tb.Text = text;
                    tb.Select(s1, s2);
                }
            }
            if (optAuto.Checked && sender != optCustom)
            {
                txtCustom.Text = Watcher.Get(createTargetText(), txtOmitDrive.Text, txtCustom.Text);
            }
            else if (optCustom.Checked && sender == optCustom)
            {
                txtCustom.Focus();
            }
        }
        /// <summary>キーロック関連チェックボックス表示制御用</summary>
        private void CheckedChanged2(object sender, EventArgs e)
        {
            XCapsLock.Enabled = WKeylock.Checked;
            XScrLk.Enabled = WKeylock.Checked;
            XNumLk.Enabled = WKeylock.Checked;
            CapsLock.Enabled = WKeylock.Checked && XCapsLock.Checked;
            ScrollLock.Enabled = WKeylock.Checked && XScrLk.Checked;
            NumLock.Enabled = WKeylock.Checked && XNumLk.Checked;
            //初期表示時以外は、フォーカス移動
            if (Timer != null)
            {
                if (sender == XCapsLock && CapsLock.Enabled) { CapsLock.Focus(); }
                else if (sender == XScrLk && ScrollLock.Enabled) { ScrollLock.Focus(); }
                else if (sender == XNumLk && NumLock.Enabled) { NumLock.Focus(); }
            }
        }
        /// <summary>フォント変更ダイアログ表示</summary>
        private void FontDialog(object sender, EventArgs e)
        {
            int index = selFont.SelectedIndex;
            if (index >= 0)
            {
                using (FontDialog dialog = new FontDialog())
                {
                    dialog.Font = IconDesign.Fonts[index];
                    dialog.MaxSize = 10;
                    dialog.MinSize = 6;
                    dialog.FixedPitchOnly = true;
                    dialog.FontMustExist = false;
                    dialog.AllowVerticalFonts = false;
                    dialog.AllowVectorFonts = true;
                    dialog.ScriptsOnly = true;
                    dialog.ShowColor = false;
                    dialog.ShowEffects = false;
                    if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        IconDesign.Fonts[index] = dialog.Font;
                        if (index == 0)
                        {
                            IconDesign.Fonts[3] = new Font(dialog.Font, (dialog.Font.Style | FontStyle.Underline));
                        }
                        selFont.SelectedIndex = index;
                        selFont.Invalidate();
                    }
                }
            }
        }
        /// <summary>色変更ダイアログ表示</summary>
        private void ColorDialog(object sender, EventArgs e)
        {
            int index = selColor.SelectedIndex;
            if (index >= 0)
            {
                using (ColorDialog dialog = new ColorDialog())
                {
                    int[] c2 = new int[IconDesign.Colors.Length + 1];
                    for (int i = 0; i < IconDesign.Colors.Length; i++)
                    {
                        c2[i] = ColorTranslator.ToWin32(IconDesign.Colors[i]);
                    }
                    c2[IconDesign.Colors.Length] = ColorTranslator.ToWin32(IconDesign.Colors[index]);
                    dialog.Color = IconDesign.Colors[index];
                    dialog.CustomColors = c2;
                    dialog.AnyColor = true;
                    dialog.FullOpen = true;
                    if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        IconDesign.Colors[index] = dialog.Color;
                        selColor.SelectedIndex = index;
                        selColor.Invalidate();
                    }
                }
            }
        }
        //ComboBox右クリック用関数：マウス位置がコントロール領域に含まれていればボタンを押す
        private bool Check(ComboBox sender, Button e)
        {
            //if (sender.Focused)
            {
                Point p = sender.PointToClient(Control.MousePosition);
                Rectangle r = sender.ClientRectangle;
                bool droppeddown = sender.DroppedDown;
                if (sender.DroppedDown) { r.Height += sender.Items.Count * sender.ItemHeight; }
                if (r.Contains(p))
                {
                    if (sender.Focused)
                    {
                        int index = sender.SelectedIndex;
                        e.PerformClick();
                        sender.DroppedDown = droppeddown;
                        sender.SelectedIndex = index;
                        return true;
                    }
                    else
                    {
                        sender.Focus();
                    }
                }
            }
            return false;
        }
        //タイマイベントでのアイコンプレビュー表示更新
        private void DrawItem(object sender, EventArgs e)
        {
            //TODO たぶんinvokeだから大丈夫だと思うが、エラーが出るなら例外で囲うこと
            if (icon == null)
            {   //Disposeされているようなら処理を行わない
                return;
            }
            //Comboboxで右クリックされているなら、ダイアログを表示する。
            if (Control.MouseButtons == System.Windows.Forms.MouseButtons.Right)
            {
                Check(selColor, btnColor);
                Check(selFont, btnFont);
            }

            //アイコンプレビューの描画処理
            int sprint = (int)TTick.Value;
            int interval = (int)TDelay.Value;
            IconDesign.FontShadow = (ushort)FontShadow.Value;
            IconDesign.IconShadow = (short)IconShadow.Value;
            char driveletter = 'D';
            if (WAggregateDisk.Checked)
            {   //ドライブ名表示を差し替え
                if (selAggregateCaption.SelectedIndex == 1) { driveletter = '1'; }
                else if (selAggregateCaption.SelectedIndex == 2) { driveletter = '0'; }
            }
            IconDesign.DrawItem(icon, driveletter, 0, drive.Check(sprint, interval), WAggregateDisk.Checked);
            IconDesign.DrawItem(icon, 'l', icon.Width / 2, driveR.Check(sprint, interval), false);
            using (Graphics g = grpDesign.CreateGraphics())
            {   //GroupBoxの左上角10ピクセルを避けて描画
                g.DrawImage(icon, 10, 0);
            }
        }
        #endregion

        #region ショートカット操作----------------------------------------------
        //ショートカットの名前
        const string SHORTCUT_NAME = "DLW";
        /// <summary>ショートカットのチェックボックスに応じた表示名・ディレクトリ・サブディレクトリ要否を返す</summary>
        private bool chkShortcut2DirInfo(object checkbox, ref string dirDispName, ref string dir, ref bool subdir)
        {
            if ((checkbox is CheckBox) == false)
            {
                return false;
            }

            if (checkbox == chkShortcutDesktop)
            {
                dirDispName = "デスクトップ";
                dir = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
                subdir = false;
                return true;
            }
            else if (checkbox == chkShortcutStartup)
            {
                dirDispName = "スタートアップ";
                dir = Environment.GetFolderPath(Environment.SpecialFolder.Startup);
                subdir = false;
                return true;
            }
            else if (checkbox == chkShortcutQuickLaunch)
            {
                dirDispName = "クイック起動バー(QuickLaunch)";
                dir = System.Environment.GetEnvironmentVariable("appdata") + @"\Microsoft\Internet Explorer\Quick Launch";
                subdir = false;
                return true;
            }
            return false;
        }
        /// <summary>該当チェックボックスに、ショートカットファイル存在有無の情報を反映する</summary>
        private void getShortcutStateIntoCheckbox(CheckBox checkbox)
        {
            string dirDispName = null;
            string dirname = null;
            bool subdir = false;
            if (chkShortcut2DirInfo(checkbox, ref dirDispName, ref dirname, ref subdir) == false)
            {
                return;
            }

            string path = null;
            CheckState ret = CheckState.Unchecked;
            try
            {
                path = ShortcutFile.ToShortcutPath(dirname, SHORTCUT_NAME, subdir);
                string linkPath = ShortcutFile.ToRealPath(path);
                if (string.IsNullOrEmpty(linkPath))
                {   //ファイル無し：チェック無し
                    ret = CheckState.Unchecked;
                }
                else if (Assembly.GetEntryAssembly().Location.Equals(linkPath, StringComparison.OrdinalIgnoreCase) == false)
                {   //ファイルはあるけどリンク先が違う：中途半端なチェック状態
                    ret = CheckState.Indeterminate;
                }
                else
                {   //OK：チェックあり
                    ret = CheckState.Checked;
                }
            }
            catch (Exception)
            {   //エラー発生：チェックボックス操作不可
                checkbox.Enabled = false;
            }
            checkbox.CheckState = ret;
        }

        /// <summary>ショートカットのチェックボックスクリック時共通イベント</summary>
        private void chkShortcut_Click(object sender, EventArgs e)
        {
            string dirDispName = null;
            string dir = null;
            bool subdir = false;
            if (chkShortcut2DirInfo(sender, ref dirDispName, ref dir, ref subdir) == false)
            {
                return;
            }

            CheckBox checkbox = (CheckBox)sender;
            if (checkbox.Checked == true)
            {
                if (MessageBox.Show(dirDispName + "にショートカットを作成します。よろしいですか？", "ショートカット作成確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    string applicationDesc = "";    //アプリケーションの概要を設定
                    foreach (Attribute attr in Assembly.GetEntryAssembly().GetCustomAttributes(true))
                    {
                        if (attr.GetType() == typeof(AssemblyDescriptionAttribute))
                        {
                            applicationDesc = ((AssemblyDescriptionAttribute)attr).Description;
                        }
                    }
                    if (subdir == true)
                    {   //サブディレクトリ可（スタートメニューのプログラムなど）の場合は、作成先サブディレクトリも確認する
                        using (FolderBrowserDialog dialog = new FolderBrowserDialog())
                        {
                            dialog.Description = "ショートカットの作成先";
                            dialog.RootFolder = Environment.SpecialFolder.Programs;
                            if (dialog.ShowDialog() != DialogResult.OK)
                            {
                                getShortcutStateIntoCheckbox(checkbox);
                                return;
                            }
                            dir = dialog.SelectedPath;
                        }
                    }
                    string ret = ShortcutFile.CreateShortcutFile(dir, SHORTCUT_NAME, Assembly.GetEntryAssembly().Location, 0, applicationDesc);
                    if (ret != null)
                    {
                        MessageBox.Show(ret, "ショートカット作成エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                }
            }
            else
            {
                if (MessageBox.Show(dirDispName + "のショートカットを削除します。よろしいですか？", "ショートカット作成確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                {
                    string ret = ShortcutFile.Delete(ShortcutFile.ToShortcutPath(dir, SHORTCUT_NAME, subdir));
                    if (ret != null)
                    {
                        MessageBox.Show(ret, "ショートカット削除エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            getShortcutStateIntoCheckbox(checkbox);
        }
        #endregion

    }
}
