﻿using System;
using System.Drawing;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using DLW.Properties;

namespace DLW
{
    /// <summary>
    /// タスクトレイに表示する監視アイコンを一括管理するクラス
    /// </summary>
    class Tray : IDisposable
    {
        //監視オブジェクトと対応するアイコン
        readonly Watcher[] watcher; //ドライブ数分用意
        readonly Icon[][][] icon; //トレイ数、左アイコン数、右アイコン数
        //タスクトレイ
        readonly NotifyIcon[] tray; //監視オブジェクト数÷２、右から先に割り当て
        internal readonly NativeWindow[] NativeWindow; //ウィンドウ（Alt+F4対策）
        //監視タイマー
        readonly System.Threading.Timer Timer;
        int interlock = 0; //再入防止用変数(interlockで操作のこと)
        //ドライブ更新監視
        readonly int TRefresh = Settings.Default.TRefresh * 1000; //秒→ミリ秒変換
        int prevRefreshTick = Environment.TickCount; //前回のドライブ更新監視
        //起動時刻・監視実行回数カウント
        readonly DateTime start = DateTime.Now;
        int refreshCount = 0; //ドライブ更新監視実行回数（経過時間計算出用）

        //コンストラクタ
        internal Tray(string message)
        {
            //監視対象ドライブ／LANを把握
            Program.Log("Tray() - 監視対象把握");
            string drive = Watcher.Get();

            //トレイアイコン(内部ウィンドウの参照も確保)＆アイコン画像生成（イベントなどはまだ仕込まない）
            Program.Log("Tray() - 描画");
            int trayCount = ((drive.Length + 1) / 2);
            tray = new NotifyIcon[trayCount];
            NativeWindow = new NativeWindow[trayCount];
            //描画用変数確保
            IconDesign IconDesign = new IconDesign();
            Icon[][][] tempicon = new Icon[trayCount][][];
            //それとNativeWindowを把握
            FieldInfo fi = typeof(NotifyIcon).GetField("window", BindingFlags.NonPublic | BindingFlags.Instance);
            //トレイアイコン表示（後ろから）
            for (int trayPos = 0; trayPos < trayCount; trayPos++)
            {   //トレイ生成、初期設定
                NotifyIcon ni = new NotifyIcon();
                tray[trayPos] = ni;
                //初期表示アイコンパターン生成、表示
                int drivePos = drive.Length - trayPos * 2 - 2;
                tempicon[trayPos] = IconDesign.Get(true, 
                    (drivePos < 0 ? (char)0 : drive[drivePos]), false, 
                    drive[drivePos + 1], false);
                Refresh(ni, tempicon[trayPos][0][0]);
                NativeWindow[trayPos] = fi.GetValue(ni) as NativeWindow;
            }
            //メッセージ表示
            if (message != null)
            {
                tray[0].ShowBalloonTip(10000, Program.FileDescription, message, ToolTipIcon.Info);
            }
            //監視オブジェクトを生成
            Program.Log("Tray() - watcher生成");
            watcher = Watcher.Get(drive);
            //アイコンに仕込むイベントを設定
            EventHandler doubleclick = Program.Show; //ダブルクリック時(バルーンクリック時も)、設定ウィンドウ表示
            MouseEventHandler mouseclick = ShowPopup;
            icon = new Icon[trayCount][][];
            //監視オブジェクトどおりにアイコンを再設定
            for (int trayPos = 0; trayPos < trayCount; trayPos++)
            {
                NotifyIcon ni = tray[trayPos];
                int drivePos = watcher.Length - trayPos * 2 - 2;
                if (drivePos < -1)
                {   //対応アイコンがなくなった場合そのまま非表示化
                    Refresh(ni, null);
                    continue;
                }
                Watcher w = (drivePos < 0 ? null : watcher[drivePos]);
                Watcher w2 = watcher[drivePos + 1];
                ni.Text = Program.ProductName + " [ " + (w == null ? "" : w.Name + " / ") + w2.Name + " ]";
                icon[trayPos] = IconDesign.Get(false, 
                    (w == null ? (char)0 : w.drive), (w is Watcher.Disk),
                    w2.drive, (w2 is Watcher.Disk));
                ni.ContextMenuStrip = Program.cms;
                ni.MouseDown += mouseclick;
                ni.BalloonTipClicked += doubleclick;
                w = (w is KeyLock ? w : (w2 is KeyLock ? w2 : null));
                if (w != null) { (w as KeyLock).NotifyIcon(ni); } //KeyLock監視アイコンに紐付け
            }
            //初回表示・タイマー設定、一時表示アイコン破棄
            Check(drive);
            Timer = new System.Threading.Timer((TimerCallback)Check, drive, 0, Settings.Default.TTick);
            Dispose(tempicon);
        }

        //クリック時のアクセス積算表示
        internal void ShowPopup(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {   //クリック時、開始時刻／起動時間、読込／書込みバイト数積算表示
                long minutes = ((long)refreshCount * TRefresh / 60000);
                StringBuilder sb = new StringBuilder();
                sb.Append(Program.FileDescription);
                sb.AppendLine();
                sb.Append(Program.isJP ? "（計測開始：" : "(Monitor start: ")
                    .Append(start.ToShortDateString())
                    .Append(" ")
                    .Append(start.ToLongTimeString())
                    .Append(" ")
                    .Append(Program.isJP ? "　監視時間計：" : "  Total: ")
                    .Append((minutes / 60).ToString("#0"))
                    .Append(Program.isJP ? "時間" : "h ")
                    .Append((minutes % 60).ToString("#0"))
                    .Append(Program.isJP ? "分）" : "min)");
                sb.AppendLine();
                sb.AppendLine();
                sb.Append("------\t")
                    .Append(Program.isJP ? "読込(受信)/書込(送信)バイト数累計" : "Read(Received)/Written(Sent) Bytes")
                    .Append("----------");
                foreach (Watcher w in this.watcher)
                {
                    if (w.drive != Watcher.WKeylock)
                    {   //キーロック以外を一覧表示
                        sb.Append(Environment.NewLine);
                        sb.Append(w.Text);
                    }
                }
                //ポップアップ表示(ContextMenuStripのTopMostにあやかる必要あり)
                if (Program.cms.Created == false)
                {   //cmsが一回も表示されてないとヘルプがトレイの裏側に隠れるので、一瞬だけ表示する
                    try
                    {
                        MethodInfo mi = typeof(NotifyIcon).GetMethod("ShowContextMenu", BindingFlags.NonPublic | BindingFlags.Instance);
                        mi.Invoke(tray[0], null);
                    }
                    catch (Exception) { }
                }
                Point p = Cursor.Position;
                int width = 12 * IconDesign.GetSystemMetrics(IconDesign.SM_CXSMICON);
                if (p.X + width > Screen.PrimaryScreen.Bounds.Width)
                {   //タスクバーを右側配置していた場合に表示しきれるように
                    p.X = Screen.PrimaryScreen.Bounds.Width - width;
                }
                Help.ShowPopup(Program.cms, sb.ToString(), p);
            }
        }

        /// <summary>監視タイマ処理</summary>
        private void Check(object drive)
        {
            if (Interlocked.CompareExchange(ref interlock, 1, 0) != 0)
            {   //すでに実行中：再入阻止
                return;
            }
            try
            {
                int tick = Environment.TickCount;
                if (tick - prevRefreshTick > TRefresh)
                {
                    //起動経過時間カウントアップ
                    refreshCount++;
                    prevRefreshTick += TRefresh;
                    if (tick - prevRefreshTick > TRefresh)
                    {   //サスペンド／休止状態になっていたと思われる。つじつまを合わせる。
                        prevRefreshTick = tick;
                        Network.Refresh(null, null); //NICの一覧も更新しておく
                    }
                    //一定時間おきにドライブ復活チェック
                    if (Watcher.Refresh())
                    {   //ドライブ変更あり
                        if (Settings.Default.TAutoRefresh && (drive as string) != Watcher.Get())
                        {   //リフレッシュ要（トレイ表示対象ドライブの増減あり）
                            Program.Refresh(null);
                            return;
                        }
                        else
                        {   //リフレッシュ否：例外オブジェクトだけクリアしておく
                            foreach (Watcher w in watcher)
                            {   //念のため破棄されていないことを確認のうえで実施：20131026
                                if (w == null)
                                {
                                    return;
                                }
                                w.Check();
                            }
                        }
                    }
                }
                int watcherLength = watcher.Length;
                for (int i = 0; i < watcherLength; i += 2)
                {
                    //各ドライブの監視状況をチェック、表示パターンを決定
                    NotifyIcon t = tray[i / 2];
                    int drivePos = watcherLength - i - 2;
                    t.Icon = icon[i / 2]
                        [(drivePos < 0 ? 0 : watcher[drivePos].Check(tick))]
                        [watcher[drivePos + 1].Check(tick)];
                }
            }
            finally
            {
                if (Interlocked.CompareExchange(ref interlock, 0, 1) != 1)
                {   //Disposeと競合しているっぽい。タイマ処理完了を伝える
                    Interlocked.Exchange(ref interlock, 2);
                }
            }
        }

        /// <summary>通知領域アイコンの表示/非表示制御（表示時は４秒制御あり）</summary>
        /// <param name="tray">通知領域アイコン</param>
        /// <param name="icon">表示するアイコン画像：非表示にするならnull</param>
        private void Refresh(NotifyIcon tray, Icon icon)
        {
            tray.Visible = false; //いったん非表示を設定する
            int index = Array.IndexOf(this.tray, tray) + 1;
            tray.Text = Program.ProductName + " #" + (index).ToString();
            tray.Icon = icon;
            if (icon != null)
            {   //2013.10.23：スタートアップ起動対策（４秒以内に表示処理が完了していないと登録に失敗してるのでリトライ）
                while (true)
                {
                    int tickCount = Environment.TickCount;
                    tray.Visible = true;
                    if (Environment.TickCount - tickCount < 4000)
                    {   //４秒以内に登録できていれば成功
                        break;
                    }
                    //失敗した場合はVisibleをfalseにしてやりなおし
                    tray.Visible = false;
                }
            }
        }

        /// <summary>リソースを破棄</summary>
        public void Dispose()
        {
            if (Timer != null)
            {   //タイマはまず最優先で止める
                Timer.Dispose();
            }
            int lockOk = Interlocked.Exchange(ref interlock, int.MaxValue);
            if (lockOk == 1)
            {   //タイマ処理中：タイマ処理の終了を待つ
                for (int i = 0; i < 500; i++)
                {
                    Thread.Sleep(2);
                    if (Interlocked.Exchange(ref interlock, int.MaxValue) == 2)
                    {   //タイマ処理の終了を検出
                        break;
                    }
                }
            }
            else if (lockOk != 0)
            {   //0でも1でもなければ、すでにdispose済のはず。脱出
                return;
            }
            //すべてのリソースを破棄
            Dispose(watcher);
            Dispose(tray);
            Dispose(icon);
        }

        /// <summary>オブジェクト破棄支援用関数</summary>
        private void Dispose<T>(T[][][] array) where T : class,IDisposable
        {
            if (array != null)
            {
                for (int i = 0; i < array.Length; i++)
                {
                    Dispose(array[i]);
                }
            }
        }
        private void Dispose<T>(T[][] array) where T : class,IDisposable
        {
            if (array != null)
            {
                for (int i = 0; i < array.Length; i++)
                {
                    Dispose(array[i]);
                }
            }
        }
        private void Dispose<T>(T[] array) where T : class, IDisposable
        {
            if (array != null)
            {
                for (int i = 0; i < array.Length; i++)
                {
                    if (array[i] is NotifyIcon)
                    {
                        Refresh((array[i] as NotifyIcon), null);
                    }
                    if (array[i] is Icon)
                    {
                        if ((array[i] as Icon).Handle != IntPtr.Zero)
                        {   //アイコンハンドルを確実に殺さないとリソース不足になることがあるため
                            DestroyIcon((array[i] as Icon).Handle);
                        }
                    }
                    if (array[i] != null)
                    {
                        try
                        {
                            array[i].Dispose();
                            //array[i] = null;
                        }
                        catch (Exception) { }
                    }
                }
            }
        }
        /// <summary>アイコン破棄はWin32APIを正しく呼ばないと問題が生じる・・・</summary>
        [DllImport("user32")]
        internal static extern bool DestroyIcon(IntPtr hIcon);
    }
}
