﻿using System.Collections.Generic;
using System.Text;

namespace DLW
{
    partial class Watcher
    {
        internal class Disk : Watcher
        {
            //集約対象
            List<Watcher> watcher;

            internal Disk(char drive, Watcher watcher)
                : base(drive)
            {
                this.watcher = new List<Watcher>() { watcher };
            }

            internal void Add(Watcher watcher)
            {
                this.watcher.Add(watcher);
            }

            /// <summary>表示名を返す</summary>
            internal override string Name
            {
                get
                {
                    StringBuilder sb = new StringBuilder(watcher.Count);
                    foreach (Watcher w in watcher)
                    {
                        sb.Append(w.drive);
                    }
                    return sb.Append(':').ToString();
                }
            }

            /// <summary>サマリー表示テキストを返す</summary>
            internal override string Text
            {
                get
                {
                    StringBuilder sb = new StringBuilder(Name);
                    sb.Append(' ');
                    sb.Append(Program.isJP ? "パーティション集約表示" : "the Same Physical Disk");
                    foreach (Watcher w in watcher)
                    {
                        sb.AppendLine();
                        sb.Append(' ', 3);
                        sb.Append(w.Text);
                    }
                    return sb.ToString();
                }
            }

            /// <summary>ステータス更新</summary>
            internal override void Check()
            {
                foreach (Watcher w in watcher)
                {
                    w.Check();
                }
            }

            /// <summary>Read/Writeチェック(各ドライブの結果を合成)</summary>
            internal override int Check(int tick)
            {
                foreach (Watcher w in watcher)
                {
                    if (w.Check(tick) > 0)
                    {   //集約対象ドライブの読み書きがあれば同様に反映
                        if (w.rd == tick) { base.rd = tick; rdCount++; }
                        if (w.wr == tick) { base.wr = tick; wrCount++; }
                    }
                    else
                    {   //集約対象ドライブに無効なものを検出：
                        return 0;
                    }
                }
                return base.Check(tick);
            }
        }
    }
}