﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using DLW.Properties;

namespace DLW
{
    /// <summary>
    /// キーロック監視クラス
    /// </summary>
    class KeyLock : Watcher
    {
        #region スタティックメンバ／メソッド------------------------------------
        //キーロックを表す定数
        //const int XCapsLock = 1;
        //const int XNumLk = 2;
        //const int XScrLk = 4;

        /// <summary>インスタンス化時にキーロック設定を適用するか</summary>
        internal static volatile bool loadSettings = true; //exe起動時点＋設定更新時のみtrue、リフレッシュ時はfalse

        /// <summary>キーロック変更操作を無効化するか</summary>
        internal static volatile ToolStripMenuItem LockDeactivated = null; //
        #endregion

        #region SendInputによるキーロック操作----------------------------------
        private static void SendInput(Keys wVk)
        {
            //そのキーを押している状態か否かを判定 
            // 「If the high-order bit is 1, the key is down; otherwise, it is up.」
            bool lockKeyPressing = ((GetKeyState((int)wVk) & 0x8000) != 0x0);
            //Shiftキー操作が必要か否かを判定
            bool needShift = ((Control.ModifierKeys & Keys.Shift) == Keys.None && wVk == Keys.CapsLock);

            Program.Log("KeyLock.SendInput(" + wVk + ")");
            INPUT[] sendData = new INPUT[(needShift ? 4 : 2)];
            int pos = 0;
            if (needShift)
            {   //Shiftキーを押す
                sendData[pos].type = 1; // INPUT_KEYBOARD;
                sendData[pos].wVk = (short)Keys.ShiftKey;
                sendData[pos].dwFlags = 0; //keydown
                pos++;
            }
            //本来目的のキーを押す
            sendData[pos].type = 1; // INPUT_KEYBOARD;
            sendData[pos].wVk = (short)wVk;
            sendData[pos].dwFlags = (lockKeyPressing ? 2 : 0); //そのキーがすでに押されているならkeyUp、否ならKeyDown
            pos++;
            //同一キーを解除する
            sendData[pos].type = 1; // INPUT_KEYBOARD;
            sendData[pos].wVk = (short)wVk;
            sendData[pos].dwFlags = (lockKeyPressing ? 0 : 2); //そのキーがすでに押されているならkeyDown、否ならKeyUp
            pos++;
            if (needShift)
            {   //Shiftキーを解除する
                sendData[pos].type = 1; // INPUT_KEYBOARD;
                sendData[pos].wVk = (short)Keys.ShiftKey;
                sendData[pos].dwFlags = 2; //keyup
                pos++;
            }
            SendInput(pos, ref sendData[0], Marshal.SizeOf(typeof(INPUT)));
        }
        [DllImport("user32")]
        private static extern void SendInput(int nInputs, ref INPUT pInputs, int cbsize);
        private struct INPUT
        {
            public int type;
            public short wVk;   //押すキーの情報を設定
            public short wScan; //mapVirtualKeyの結果を設定
            public int dwFlags; //0=KeyDown, 2=KeyUp。いずれも拡張キー指定を
            public int time;    //デフォルトの0固定でよい
            public IntPtr dwExtraInfo; //とくに設定不要
            public int filler; //8byte分余白を取らないとだめらしい
            public int filler2; //8byte分余白を取らないとだめらしい
        };
        [DllImport("user32")]
        private static extern int GetKeyState(int vKey);
        #endregion

        #region インスタンスメンバ／メソッド------------------------------------
        //キーロックメニュー
        ContextMenuStrip cms;
        ToolStripMenuItem CapsLock;
        ToolStripMenuItem Scroll;
        ToolStripMenuItem NumLock;

        //TooltipText書き換え対象NotifyIcon、および本来の文字列
        NotifyIcon icon;
        //NotifyIconの本来のテキスト内容
        string ToolTipText;

        /// <summary>コンストラクタ</summary>
        internal KeyLock()
            : base(Watcher.WKeylock)
        {   //メニュー表示初期化
            cms = new ContextMenuStrip();
            //メニュー項目を作成しながらキーロック状態固定時のロック状態変更を実施。
            cms.Items.Add(CapsLock = Items("Caps Lock", "Shift+CapsLock", Watcher.XCapsLock, Keys.CapsLock));
            cms.Items.Add(Scroll = Items("Scroll Lock", "Scroll", Watcher.XScrLk, Keys.Scroll));
            cms.Items.Add(NumLock = Items("Num Lock", "NumLock", Watcher.XNumLk, Keys.NumLock));
            if (Contains(Watcher.XCapsLock) || Contains(Watcher.XScrLk) || Contains(Watcher.XNumLk))
            {   //キーロック操作制限ありなら、一時解除メニュー項目も生成
                cms.Items.Add(new ToolStripSeparator() { Enabled = false });
                bool lockChecked = (LockDeactivated != null && LockDeactivated.Checked);
                cms.Items.Add(LockDeactivated = new ToolStripMenuItem("メニュー以外での操作制限を一時解除(&R)")
                {
                    CheckOnClick = true,
                    Checked = lockChecked,
                    ToolTipText = "相性の悪いソフト(仮想端末等)の使用中などに一時解除してください。"
                });
            }
            else
            {   //キーロック操作制限なしなら、一時解除メニューなし
                LockDeactivated = null;
            }
            loadSettings = false;
            //初期状態を反映
            rdCount = IsKeyLocked();
        }
        //メニュー項目生成支援 ＆ キーロック値初期設定
        private ToolStripMenuItem Items(string text, string ShortcutKeyDisplayString, char drive, Keys wVk)
        {
            bool locked = Contains(drive);
            ToolStripMenuItem i = new ToolStripMenuItem(text + "(&" + text.Substring(0, 1) + ")");
            i.ShortcutKeyDisplayString = (locked ? "<制限中>" + ShortcutKeyDisplayString : ShortcutKeyDisplayString);
            i.ToolTipText = text + "キーのロック状態を変更します。";
            if (locked)
            {
                i.ToolTipText += Environment.NewLine + "（現在、メニュー操作以外での変更を無効にしています）";
                int index = IndexOf(drive);
                if (index != 0 && loadSettings)
                {   //キーロック固定指定ありならばキーロック設定値を適用
                    bool lockvalue = (index == 1);
                    Check(wVk, lockvalue);
                    i.Checked = lockvalue;
                }
                else
                {
                    i.Checked = IsKeyLocked(wVk);
                }
            }
            i.Click += (object sender, EventArgs e) =>
            {   //クリックされたアイテムのキーロック変更
                SendInput(wVk);
                Refresh(sender, e);
            };
            return i;
        }

        /// <summary>状態表示する通知アイコンを紐付ける</summary>
        internal void NotifyIcon(NotifyIcon icon)
        {   //キーロック内容反映対象NotifyIconおよびテキストを把握
            this.icon = icon;
            this.ToolTipText = icon.Text;
            this.Replace();
            //メインメニューに紐付ける
            Program.KeyLock.Visible = true;
            Program.KeyLock.DropDown = cms;
            //イベント設定
            cms.Closing += (sender, e) =>
            {   //クリック操作時はメニューを閉じない
                e.Cancel = (e.CloseReason == ToolStripDropDownCloseReason.ItemClicked);
            };
            cms.Opening += Refresh;
            cms.KeyDown += Refresh;
            cms.KeyUp += Refresh;
        }
        private void Refresh(object sender, EventArgs e)
        {   //キーロックメニューのチェック表示を更新
            Program.Log("KeyLock.Refresh()");
            CapsLock.Checked = (IsKeyLocked(Keys.CapsLock));
            NumLock.Checked = (IsKeyLocked(Keys.NumLock));
            Scroll.Checked = (IsKeyLocked(Keys.Scroll));
        }

        private void Replace()
        {   //トレイアイコンツールチップの表示更新
            if (icon != null)
            {
                icon.Text = ToolTipText.Replace(Name,
                    "Caps:" + (IsKeyLocked(Keys.CapsLock) ? "ON" : "off")
                    + ", ScrLk:" + (IsKeyLocked(Keys.Scroll) ? "ON" : "off")
                    + ", NumLk:" + (IsKeyLocked(Keys.NumLock) ? "ON" : "off"));
            }
        }

        //破棄
        public override void Dispose()
        {   //キーロックメニューを破棄
            Program.KeyLock.Visible = false;
            Program.KeyLock.DropDown = null;
            cms.Dispose();
        }

        /// <summary>read/write有無チェック:監視結果に応じた値を返す</summary>
        internal override int Check(int tick)
        {
            int ret = IsKeyLocked();
            if (ret != rdCount)
            {   //前回チェック時とキーロック状態に変更あり
                rdCount = ret;
                if (LockDeactivated != null)
                {   //メニュー指定のキーロック状態と相違ないかチェック
                    Program.cms.Invoke((MethodInvoker)(() =>
                    {   //キーロックメニュー非表示かつ制限解除されていなければ、固定チェック＆反転
                        if (cms.Visible == false && LockDeactivated.Checked == false)
                        {
                            if ((Contains(Watcher.XNumLk) && Check(Keys.NumLock, NumLock.Checked))
                                | (Contains(Watcher.XScrLk) && Check(Keys.Scroll, Scroll.Checked))
                                | (Contains(Watcher.XCapsLock) && Check(Keys.CapsLock, CapsLock.Checked)))
                            {   //次回念のため無効化キー再チェックできるような値を設定
                                rdCount = int.MinValue;
                            };
                        }
                    }));
                }
                //ロック状態変更ありなら、NotifyIconのToolTipを書き換え
                this.Replace();
            }
            return ret;
        }
        /// <summary>キーロック状態に応じた値を返す</summary>
        private int IsKeyLocked()
        {
            int ret = 0;
            if (IsKeyLocked(Keys.CapsLock)) { ret += 1; }
            if (IsKeyLocked(Keys.NumLock)) { ret += 2; }
            if (IsKeyLocked(Keys.Scroll)) { ret += 4; }
            return ret;
        }

        /// <summary>引数指定のキーがロック状態ならtrueを返す(Control.IsKeyLockedとはことなりインジゲータ表示通りの値を返す)</summary>
        private static bool IsKeyLocked(Keys wVk)
        {   //Control.IsKeyLockedでは0x8001で見ているが、LED点灯状態と同期をとるため0x0001でチェックする。 
            // 「If the low-order bit is 1, the key is toggled.」とのこと
            return ((GetKeyState((int)wVk) & 0x0001) != 0x0);
        }

        /// <summary>引数指定のkeyについて、ロック状態がvalueのとおりでなければ反転のためのキー送信を行う</summary>
        private bool Check(Keys wVk, bool value)
        {
            if (IsKeyLocked(wVk) == value)
            {   //キーロックの状態は想定どおりである。キー送信せず
                return false;
            }
            //キーロック状態が想定通りではない。キー送信＆結果確認
            SendInput(wVk);
            if (IsKeyLocked(wVk) == value)
            {   //キーロック状態変更成功
                return true;
            }
            //キーロック状態変更のイベント消化がまだである可能性をチェック
            Application.DoEvents();
            if (IsKeyLocked(wVk) == value)
            {   //キーロック状態変更成功
                return true;
            }
            //これでもだめなら高権限ウィンドウにフォーカスが当たっている可能性にかけてcmsに対しキー送信する
            IntPtr currentWindow = Program.GetForegroundWindow();
            Program.SetForegroundWindow(Program.cms.Handle);
            SendInput(wVk);
            if (Program.cms.Visible == false)
            {   //キー送信後、アクティブウィンドウを元に戻す
                Program.SetForegroundWindow(currentWindow);
            }
            return true;
        }
        #endregion
    }
}